@@ -29,7 +29,7 @@ public:
 	virtual ~NodePropertiesDlg();
 	virtual void Create();
 
-	const char *GetNodeTitle();
+	std::string GetNodeTitle();
 
 	int GetIconType();
 	void SetIconType(int nType);
