@@ -12,7 +12,9 @@
  #endif
  #include "_win/bootstart.h" //register program loading at boot time
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
  #include <io.h> //access
  #define access _access
@@ -30,6 +32,8 @@
 #include "interface.h"
 #include "gui/FileDialog.h"
 #include "lib/IniFile.h"
+#include <stdlib.h>
+#include <gdk/gdkkeysyms.h>
 
 #define ICON_NONE    -2
 #define ICON_CUSTOM  -1
@@ -51,10 +55,13 @@ extern bool g_bExportDataAsSeparateFile;
 extern IniFile g_objIni;
 extern bool g_bTreeToTheRight;
 extern GdkColor g_linkColor;
+extern bool g_bWrapTree;
 
 int g_nDefaultNodeIconType = ICON_NONE;
 std::string g_strDefaultIconFile;
 
+extern GdkColor g_rgbTextViewBkColor;
+
 void restart_autosave();
 void restart_enc_unload_timer();
 static void on_autosave_checked (GtkMenuItem *menuitem, gpointer user_data);
@@ -69,13 +76,17 @@ static void on_icon_internal_clicked (GtkMenuItem *menuitem, gpointer user_data)
 static void on_icon_custom_clicked (GtkMenuItem *menuitem, gpointer user_data);
 static void on_xpm_select_clicked (GtkMenuItem *menuitem, gpointer user_data);
 static void on_internal_combo_selected (GtkComboBox *widget, gpointer user_data);
+static gboolean on_numeric_entry_keypress(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
+static gboolean on_numeric_entry_changed(GtkWidget *widget, GdkEventFocus *event, gpointer user_data);
+
+void update_tree_wrap();
 
 void RefreshTextViewCaretColor(GdkColor &BkColor);
 const char *combo_get_text(GtkWidget *cbo3);
 const char *InternalIcon_Index2Name(int nIndex);
 const char **InternalIcon_GetFromIdx(int);
 int InternalIcon_Name2Index(const char *);
-
+bool copy_file(const char *szFrom, const char *szTo);
 
 OptionsDialog::OptionsDialog()
 {
@@ -105,6 +116,8 @@ void OptionsDialog::OnDialogInit()
 	GtkWidget *reloadlast_btn = lookup_widget(m_pDialog, "reloadlast_btn");
 	GtkWidget *regformats_btn = lookup_widget(m_pDialog, "regformats_btn");
 	GtkWidget *wraptext_btn	  = lookup_widget(m_pDialog, "wraptext_btn");
+	GtkWidget *wraptree_btn	  = lookup_widget(m_pDialog, "wraptree_btn");
+	GtkWidget *treelines_btn  = lookup_widget(m_pDialog, "treelines_btn");
 	GtkWidget *fontbutton2	  = lookup_widget(m_pDialog, "fontbutton2");
 	GtkWidget *fontbutton3	  = lookup_widget(m_pDialog, "fontbutton3");
 	GtkWidget *comboboxentry1 = lookup_widget(m_pDialog, "comboboxentry1");
@@ -166,6 +179,14 @@ void OptionsDialog::OnDialogInit()
 	if(g_objIni.GetValue("Display", "WrapText", bWrapText))
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wraptext_btn), bWrapText);
 
+	bool bWrapTree;
+	if(g_objIni.GetValue("Display", "WrapTree", bWrapTree))
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wraptree_btn), bWrapTree);
+
+	bool bShowTreeLines;
+	if(g_objIni.GetValue("Display", "ShowTreeLines", bShowTreeLines))
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(treelines_btn), bShowTreeLines);
+
 	bool bMin2Tray;
 	if(g_objIni.GetValue("Display", "MinimizeToTray", bMin2Tray))
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(min2tray_btn), bMin2Tray);
@@ -349,6 +370,8 @@ void OptionsDialog::OnDialogOK()
 	GtkWidget *reloadlast_btn = lookup_widget(m_pDialog, "reloadlast_btn");
 	GtkWidget *regformats_btn = lookup_widget(m_pDialog, "regformats_btn");
 	GtkWidget *wraptext_btn	  = lookup_widget(m_pDialog, "wraptext_btn");
+	GtkWidget *wraptree_btn	  = lookup_widget(m_pDialog, "wraptree_btn");
+	GtkWidget *treelines_btn  = lookup_widget(m_pDialog, "treelines_btn");
 	GtkWidget *fontbutton2	  = lookup_widget(m_pDialog, "fontbutton2");
 	GtkWidget *fontbutton3	  = lookup_widget(m_pDialog, "fontbutton3");
 	GtkWidget *comboboxentry1 = lookup_widget(m_pDialog, "comboboxentry1");
@@ -390,6 +413,22 @@ void OptionsDialog::OnDialogOK()
 			//register program loading at boot time
 			if(!IsBootKeySet("notecase"))
 				RunProgramAtBoot("notecase", GetAppPath().c_str(), TRUE);	//set
+		#else
+			//register program loading at boot time
+			std::string strDir = GetHomeDir();
+			EnsureTerminated(strDir, '/');
+			strDir += ".config/autostart/";
+			EnsureDirExists(strDir);
+			strDir += "notecase.desktop";
+
+			std::string strSrcFile;
+			#ifdef _NOKIA_MAEMO
+				strSrcFile = "/usr/share/applications/hildon/notecase.desktop";
+			#else
+				strSrcFile = "/usr/share/applications/notecase.desktop";
+			#endif
+			//copy file
+			copy_file(strSrcFile.c_str(), strDir.c_str()); //TOFIX error report
 		#endif
 	}
 	else
@@ -399,6 +438,14 @@ void OptionsDialog::OnDialogOK()
 			//unregister program loading at boot time
 			if(IsBootKeySet("notecase"))
 				RunProgramAtBoot("notecase", GetAppPath().c_str(), FALSE);	//remove
+		#else
+			//unregister program loading at boot time
+			std::string strDir = GetHomeDir();
+			EnsureTerminated(strDir, '/');
+			strDir += ".config/autostart/";
+			EnsureDirExists(strDir);
+			strDir += "notecase.desktop";
+			remove(strDir.c_str());
 		#endif
 	}
 
@@ -444,6 +491,30 @@ void OptionsDialog::OnDialogOK()
 		}
 	}
 
+	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(wraptree_btn)))
+	{
+		g_objIni.SetValue("Display", "WrapTree", 1);
+		g_bWrapTree=true;
+		update_tree_wrap();
+	}
+	else
+	{
+		g_objIni.SetValue("Display", "WrapTree", 0);
+		g_tree.WrapTreeToSize(-1);
+		g_bWrapTree=false;
+	}
+
+	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(treelines_btn)))
+	{
+		g_objIni.SetValue("Display", "ShowTreeLines", 1);
+		g_tree.SetTreeLines(true);
+	}
+	else
+	{
+		g_objIni.SetValue("Display", "ShowTreeLines", 0);
+		g_tree.SetTreeLines(false);
+	}
+
 #if GTK_CHECK_VERSION(2,4,0) //define minimal version for this api
 	//set fonts
 	const gchar *szFont1 = gtk_font_button_get_font_name(GTK_FONT_BUTTON(fontbutton2));
@@ -469,11 +540,13 @@ void OptionsDialog::OnDialogOK()
 	szColor = g_strdup_printf ("#%04x%04x%04x", color1.red, color1.green, color1.blue);
 	g_objIni.SetValue("Display", "BackgroundColor", szColor);
 	g_free(szColor);
+	g_rgbTextViewBkColor = color1;
 	
 	//update widgets with new color
 	gtk_widget_modify_text(GTK_WIDGET(textview), GTK_STATE_NORMAL, &color);
 	gtk_widget_modify_text(GTK_WIDGET(treeview), GTK_STATE_NORMAL, &color);
 	gtk_widget_modify_base(GTK_WIDGET(textview), GTK_STATE_NORMAL, &color1);
+	gtk_widget_modify_base(GTK_WIDGET(textview), GTK_STATE_INSENSITIVE, &color1);
 	gtk_widget_modify_base(GTK_WIDGET(treeview), GTK_STATE_NORMAL, &color1);
 	RefreshTextViewCaretColor(color1);
 
@@ -730,6 +803,9 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	GtkWidget *checkbutton22;
 	GtkWidget *checkbutton23;
 	GtkWidget *checkbutton24;
+	GtkWidget *checkbutton27;
+	GtkWidget *checkbutton30;
+
 //
 	GtkWidget *button_xpm_select;
 	GtkWidget *image1;
@@ -782,9 +858,6 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	checkbutton0 = gtk_check_button_new_with_mnemonic (_("Start at logon"));
 	gtk_widget_show (checkbutton0);
 	gtk_box_pack_start (GTK_BOX (vbox4), checkbutton0, FALSE, FALSE, 0);
-#ifndef _WIN32
-	gtk_widget_set_sensitive (checkbutton0, FALSE); //TOFIX temp disable until implemented
-#endif
 
 	checkbutton1 = gtk_check_button_new_with_mnemonic (_("Maximize on startup"));
 	gtk_widget_show (checkbutton1);
@@ -793,9 +866,6 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	checkbutton2 = gtk_check_button_new_with_mnemonic (_("Allow single instance only"));
 	gtk_widget_show (checkbutton2);
 	gtk_box_pack_start (GTK_BOX (vbox4), checkbutton2, FALSE, FALSE, 0);
-#ifndef _WIN32
-	gtk_widget_set_sensitive (checkbutton2, FALSE); //TOFIX temp disable until implemented
-#endif
 	
 	checkbutton3 = gtk_check_button_new_with_mnemonic (_("Reload last used document"));
 	gtk_widget_show (checkbutton3);
@@ -882,7 +952,15 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 
 	checkbutton5 = gtk_check_button_new_with_mnemonic (_("Wrap text"));
 	gtk_widget_show (checkbutton5);
-	gtk_table_attach (GTK_TABLE (table1), checkbutton5, 0, 2, 7, 8, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+	gtk_table_attach (GTK_TABLE (table1), checkbutton5, 0, 1, 7, 8, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+
+	checkbutton27 = gtk_check_button_new_with_mnemonic (_("Wrap titles tree"));
+	gtk_widget_show (checkbutton27);
+	gtk_table_attach (GTK_TABLE (table1), checkbutton27, 1, 2, 7, 8, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+
+	checkbutton30 = gtk_check_button_new_with_mnemonic (_("Show tree lines"));
+	gtk_widget_show (checkbutton30);
+	gtk_table_attach (GTK_TABLE (table1), checkbutton30, 2, 3, 7, 8, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
 	label3 = gtk_label_new (_("Display"));
 	gtk_widget_show (label3);
@@ -950,7 +1028,7 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	gtk_widget_show (table2);
 	gtk_container_add (GTK_CONTAINER (notebook1), table2);
 
-	checkbutton8 = gtk_check_button_new_with_mnemonic (_("Autosave document every:"));
+	checkbutton8 = gtk_check_button_new_with_mnemonic (_("Autosave to temporary crash-protection copy every:"));
 	gtk_widget_show (checkbutton8);
 	gtk_table_attach (GTK_TABLE (table2), checkbutton8, 0, 2, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
 
@@ -958,6 +1036,8 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	gtk_widget_show (entry1);
 	gtk_widget_set_sensitive (entry1, FALSE);
 	gtk_table_attach (GTK_TABLE (table2), entry1, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 2, 0);
+	g_signal_connect (entry1, "key-press-event", G_CALLBACK (on_numeric_entry_keypress), this);
+	g_signal_connect (entry1, "focus-out-event", G_CALLBACK (on_numeric_entry_changed), GINT_TO_POINTER(30));
 
 	label20 = gtk_label_new (_("seconds"));
 	gtk_widget_show (label20);
@@ -977,6 +1057,8 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	gtk_widget_show (entry5);
 	gtk_widget_set_sensitive (entry5, FALSE);
 	gtk_table_attach (GTK_TABLE (table2), entry5, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 2, 0);
+	g_signal_connect (entry5, "key-press-event", G_CALLBACK (on_numeric_entry_keypress), this);
+	g_signal_connect (entry5, "focus-out-event", G_CALLBACK (on_numeric_entry_changed), GINT_TO_POINTER(1));
 
 	label20 = gtk_label_new (_("backup files"));
 	gtk_widget_show (label20);
@@ -1007,6 +1089,8 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	gtk_widget_show (entry3);
 	gtk_widget_set_sensitive (entry3, FALSE);
 	gtk_table_attach (GTK_TABLE (table3), entry3, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 2, 0);
+	g_signal_connect (entry3, "key-press-event", G_CALLBACK (on_numeric_entry_keypress), this);
+	g_signal_connect (entry3, "focus-out-event", G_CALLBACK (on_numeric_entry_changed), GINT_TO_POINTER(1));
 
 	checkbutton11 = gtk_check_button_new_with_mnemonic (_("Export link to image file as the image tag"));
 	gtk_widget_show (checkbutton11);
@@ -1051,6 +1135,8 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	gtk_widget_show (entry4);
 	gtk_widget_set_sensitive (entry4, FALSE);
 	gtk_table_attach (GTK_TABLE (table1), entry4, 0, 1, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 2, 0);
+	g_signal_connect (entry4, "key-press-event", G_CALLBACK (on_numeric_entry_keypress), this);
+	g_signal_connect (entry4, "focus-out-event", G_CALLBACK (on_numeric_entry_changed), GINT_TO_POINTER(1));
 
 	label20 = gtk_label_new (_("minutes"));
 	gtk_widget_show (label20);
@@ -1175,6 +1261,8 @@ GtkWidget* OptionsDialog::create_options_dialog (GtkWidget* parent)
 	GLADE_HOOKUP_OBJECT (Options, checkbutton3, "reloadlast_btn");
 	GLADE_HOOKUP_OBJECT (Options, checkbutton4, "regformats_btn");
 	GLADE_HOOKUP_OBJECT (Options, checkbutton5, "wraptext_btn");
+	GLADE_HOOKUP_OBJECT (Options, checkbutton27, "wraptree_btn");
+	GLADE_HOOKUP_OBJECT (Options, checkbutton30, "treelines_btn");
 	GLADE_HOOKUP_OBJECT (Options, checkbutton6, "min2tray_btn");
 	GLADE_HOOKUP_OBJECT (Options, checkbutton7, "last_position_btn");
 	GLADE_HOOKUP_OBJECT (Options, checkbutton19, "close2tray_btn");
@@ -1514,3 +1602,42 @@ void on_internal_combo_selected (GtkComboBox *widget, gpointer user_data)
 	pDlg->UpdatePreview();
 }
 
+gboolean on_numeric_entry_keypress(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+{
+	if( event->type == GDK_KEY_PRESS &&
+		((gdk_keyval_to_unicode (event->keyval) >= '0' && gdk_keyval_to_unicode (event->keyval) <= '9') || 
+		event->keyval == GDK_Left  ||
+		event->keyval == GDK_Right ||
+		event->keyval == GDK_Home  ||
+		event->keyval == GDK_End   ||
+		event->keyval == GDK_Delete	||
+		event->keyval == GDK_BackSpace) )
+		return FALSE; //keep processing
+
+	return TRUE;	
+}
+
+gboolean on_numeric_entry_changed(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
+{
+	int nMinValue = GPOINTER_TO_INT(user_data);
+
+	//OptionsDialog *pDlg = (OptionsDialog *)user_data;
+	std::string strValue = gtk_entry_get_text(GTK_ENTRY(widget));
+	if(!strValue.empty())
+	{
+		int nValue = atoi(strValue.c_str());
+		if(nValue < nMinValue)
+		{
+			char szBuffer[30];
+			sprintf(szBuffer, "%d", nMinValue);
+			gtk_entry_set_text(GTK_ENTRY(widget), szBuffer);
+
+			char szMessage[1030];
+			sprintf(szMessage, _("Minimal value is %d sec!"), nMinValue);
+			gtkMessageBox(szMessage);
+			return TRUE;
+		}
+	}
+
+	return FALSE;	//propagate
+}
