@@ -71,7 +71,7 @@ void TreeView::Create()
 
 	gtk_tree_view_set_headers_visible((GtkTreeView *)treeview1, FALSE);
 	gtk_tree_view_set_enable_search((GtkTreeView *)treeview1, FALSE); 
-	g_object_set (treeview1, "level_indentation", 10, NULL);
+	//g_object_set (treeview1, "level_indentation", 10, (char *)NULL);
 	gtk_widget_show (treeview1);
 
 	g_signal_connect (treeview1, "button_release_event", G_CALLBACK (treeview1_popup_menu_event_handler), NULL);
@@ -93,11 +93,11 @@ void TreeView::Create()
 
 	renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
-	gtk_tree_view_column_set_attributes(column, renderer, "pixbuf", STORE_IDX_ICON, NULL);
+	gtk_tree_view_column_set_attributes(column, renderer, "pixbuf", STORE_IDX_ICON, (char *)NULL);
 
 	renderer = gtk_cell_renderer_text_new();
-	g_object_set(renderer, "underline-set",  TRUE, NULL);
-	g_object_set(renderer, "foreground-set", TRUE, NULL);
+	g_object_set(renderer, "underline-set",  TRUE, (char *)NULL);
+	g_object_set(renderer, "foreground-set", TRUE, (char *)NULL);
 
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
 	gtk_tree_view_column_set_attributes(column, renderer, 
@@ -105,7 +105,7 @@ void TreeView::Create()
 			"strikethrough",  STORE_IDX_STRIKE, 
 			"underline",      STORE_IDX_UNDERLINE,
 			"foreground-gdk", STORE_IDX_COLOR,
-			NULL);
+			(char *)NULL);
 
 	g_signal_connect(renderer, "edited", (GCallback) cell_edited_callback, this);
 	g_signal_connect(renderer, "editing-started", (GCallback) cell_edited_start_callback, this);
@@ -148,9 +148,9 @@ void TreeView::EditLabel()
 		GList *list1 = gtk_tree_view_column_get_cell_renderers(column1);
 		GtkCellRendererText *renderer = (GtkCellRendererText *)g_list_nth_data(list1, SCREEN_IDX_TEXT);
 		GtkTreePath* path1 = gtk_tree_model_get_path(model, &iter);
-		g_object_set(renderer, "editable", TRUE, NULL);	//makes tree cells editable
+		g_object_set(renderer, "editable", TRUE, (char *)NULL);	//makes tree cells editable
 		gtk_tree_view_set_cursor(treeview, path1, column1, TRUE);	//trigger 
-		g_object_set(renderer, "editable", FALSE, NULL);//makes tree cells non-editable
+		g_object_set(renderer, "editable", FALSE, (char *)NULL);//makes tree cells non-editable
 		g_list_free(list1);
 		gtk_tree_path_free(path1);
 	}
@@ -651,12 +651,12 @@ void TreeView::WrapTreeToSize(int nSize)
 	g_object_set (G_OBJECT (renderer), 
 		"wrap-mode", PANGO_WRAP_CHAR,
 		"wrap-width", nSize, 
-		NULL);
+		(char *)NULL);
 }
 
 void TreeView::SetTreeLines(bool bTree)
 {
 	g_object_set (G_OBJECT (m_pWidget),
 		"enable-tree-lines", bTree,
-		NULL);
+		(char *)NULL);
 }
