@@ -318,7 +318,7 @@ void GuiLanguage::Initialize(const char *szIsoCode)
 		}
 
 		//set language localisation for this program
-		putenv( g_strconcat("LANG=", szLocaleCode, NULL) );
+		putenv( g_strconcat("LANG=", szLocaleCode, (char *)NULL) );
 		setlocale(LC_ALL, "");
 
 		m_nLangIdx = GetLocaleTableIdx(szIsoCode);
