@@ -23,7 +23,9 @@
  #include <unistd.h>
 #else
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
 #endif 
 
@@ -32,6 +34,7 @@
 #include "lib/IniFile.h"
 #include "lib/FilePath.h"
 #include "lib/DocActionManager.h"
+#include "lib/File64.h"
 #include "DocAction.h"
 #include "mru.h"
 
@@ -51,7 +54,6 @@
 #include "../res/internal_lock.xpm"
 #include "../res/internal_new_dir.xpm"
 #include "../res/internal_recycle.xpm"
-#include "../res/page_link.xpm"
 
 #ifdef _WIN32
  #include "_win/RegisterExtension.h"
@@ -79,6 +81,8 @@ void register_extensions_check();
 
 void cell_edited_callback(GtkCellRendererText *cell, gchar *path_string, gchar *new_text, gpointer user_data);
 void on_MRU_activate (GtkMenuItem *menuitem, gpointer user_data);
+void treeview_show_popup_menu();
+bool copy_file(const char *szFrom, const char *szTo);
 
 //
 // MRU list
@@ -86,175 +90,6 @@ void on_MRU_activate (GtkMenuItem *menuitem, gpointer user_data);
 MRU g_objMRU;
 GtkWidget *g_menuitem5;
 
-void treeview_show_popup_menu()
-{
-	GtkWidget *menu, *submenu;
-	GtkWidget *menu_item, *move_item, *sort_item, *icon;
-	int button, event_time;
-
-	menu = gtk_menu_new ();
-	//g_signal_connect (menu, "deactivate", G_CALLBACK(gtk_widget_destroy), NULL);
-
-	GtkAccelGroup *accel_group = gtk_accel_group_new();
-	gtk_menu_set_accel_group (GTK_MENU (menu), accel_group);
-
-	// ... add menu items with accelerators ... 
-	menu_item = gtk_menu_item_new_with_label(_("Insert Node"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_insert_node), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Insert, (GdkModifierType)0, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_menu_item_new_with_label(_("Insert Child Node"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_insert_child_node), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Insert, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Delete Node"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_delete_node), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget
-	icon = gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Delete, (GdkModifierType)0, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_menu_item_new_with_label(_("Rename Node"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_rename_node), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_F2, (GdkModifierType)0, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, 'M', (GdkModifierType)GDK_CONTROL_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	move_item = gtk_menu_item_new_with_label(_("Move Node"));
-	gtk_menu_append(menu, move_item);
-	gtk_widget_show (move_item);  // Show the widget 
-
-	sort_item = gtk_menu_item_new_with_label(_("Sort"));
-	gtk_menu_append(menu, sort_item);
-	gtk_widget_show (sort_item);  // Show the widget 
-
-	menu_item = gtk_menu_item_new_with_label(_("Expand All"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_expand_all), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-
-	menu_item = gtk_menu_item_new_with_label(_("Collapse All"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_collapse_all), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Mark node as finished"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_node_toggle_finished), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Node Properties"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_node_properties), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Return, (GdkModifierType)GDK_CONTROL_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Link wizard"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_link_wizard_activate), NULL);
-	gtk_menu_append(menu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	GdkPixbuf* pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **)&page_link);
-	icon = gtk_image_new_from_pixbuf (pixbuf);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-
-	//"move node" submenu
-	submenu = gtk_menu_new ();
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Up"));
-	gtk_menu_append(submenu, menu_item);
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_up), NULL);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Up, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Down"));
-	gtk_menu_append(submenu, menu_item);
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_down), NULL);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Down, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Left"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_left), NULL);
-	gtk_menu_append(submenu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Left, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Right"));
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_right), NULL);
-	gtk_menu_append(submenu, menu_item);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-	gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Right, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (move_item), submenu);
-
-	//"sort node" submenu
-	submenu = gtk_menu_new ();
-
-	menu_item = gtk_image_menu_item_new_with_label(_("_Sort Children Ascending"));
-	gtk_menu_append(submenu, menu_item);
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_child_ascending), NULL);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Sort _Children Descending"));
-	gtk_menu_append(submenu, menu_item);
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_child_descending), NULL);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_SORT_DESCENDING, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Sort Root Ascending"));
-	gtk_menu_append(submenu, menu_item);
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_root_ascending), NULL);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-
-	menu_item = gtk_image_menu_item_new_with_label(_("Sort Root Descending"));
-	gtk_menu_append(submenu, menu_item);
-	g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_root_descending), NULL);
-	gtk_widget_show (menu_item);  // Show the widget 
-	icon = gtk_image_new_from_stock (GTK_STOCK_SORT_DESCENDING, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (icon);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), icon);
-
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (sort_item), submenu);
-
-//	if (event)
-//		event_time = event->time;
-//	else
-		event_time = gtk_get_current_event_time ();
-	button = 0;	//FIX: allow mouse button to trigger the submenu
-
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, event_time);
-}
-
 gboolean treeview1_popup_menu_event_handler (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
 	// Ignore double-clicks and triple-clicks
@@ -370,8 +205,11 @@ void cell_edited_callback(GtkCellRendererText *cell, gchar *path_string, gchar *
 
 void on_textview_edited( GtkTextBuffer *widget, gpointer data)
 {
-	g_doc.SetModified(true);
-	RefreshMainTitle();
+	bool bModified = g_doc.IsModified();
+	if(!bModified){
+		g_doc.SetModified(true);
+		RefreshMainTitle();
+	}
 }
 
 //
@@ -500,7 +338,32 @@ void register_extensions_check()
 		reg.UpdateRegistry();
 
 #else
-	//TOFIX linux version missing
+	//install the formats into the .local database
+	std::string strMime = GetHomeDir();
+	EnsureTerminated(strMime, '/');
+	strMime += ".local/share/mime/";
+
+	std::string strDir = strMime + "packages/";
+	EnsureDirExists(strDir);
+
+	strDir = strMime + "packages/";
+	EnsureDirExists(strDir);
+	strDir += "notecase.xml";
+
+	std::string strSrcFile;
+	#ifdef _NOKIA_MAEMO
+		strSrcFile = "/usr/share/mime/packages/notecase.xml";
+	#else
+		strSrcFile = "/usr/share/mime/packages/notecase.xml";
+	#endif
+
+	//copy file
+	copy_file(strSrcFile.c_str(), strDir.c_str()); //TOFIX error report
+
+	//update the system
+	std::string strCmd = "update-mime-database " + strMime;
+	/*bool bOK = */g_spawn_command_line_async(strCmd.c_str(), NULL);
+	//TOFIX call desktop-file-utils?
 #endif
 }
 
@@ -810,3 +673,23 @@ bool IsPIDRunning(unsigned int nPID)
 
 	return true;
 }
+
+bool copy_file(const char *szFrom, const char *szTo)
+{
+	//copy file
+	File64 fileSrc, fileDst;
+	if(!fileSrc.Open(szFrom, F64_READ|F64_SHARE_READ|F64_OPEN_EXISTING)){
+		TRACE("copy_file: fail to open file [%s]\n", szFrom);
+		return false;
+	}
+	if(!fileDst.Open(szTo, F64_WRITE|F64_SHARE_READ|F64_OPEN_NEW)){
+		TRACE("copy_file: fail to create file [%s]\n", szTo);
+		return false;
+	}
+	int nRead = 0;
+	char szBuffer[1024];
+	while((nRead = fileSrc.Read(szBuffer, sizeof(szBuffer))) > 0){
+		fileDst.Write(szBuffer, nRead);
+	}
+	return true;
+}
