@@ -19,6 +19,7 @@
 #include <gtk/gtk.h>
 #include "IniFile.h"
 #include "Base64.h"
+#include "../gui/ProgressDlg.h"
 
 #include "../config.h"
 #include "../support.h"	//TOFIX dependent on non-core
@@ -27,7 +28,9 @@
  #include <strings.h>	//strcasecmp
 #else
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
  #define strcasecmp stricmp
 #endif
@@ -49,6 +52,8 @@ static bool g_bUseTOC = false;
 extern IniFile g_objIni;
 extern bool g_bExportDataAsSeparateFile;
 
+static std::string g_strTabReplacement;
+
 int InternalIcon_Name2Index(const char *szName);
 const char *InternalIcon_Index2Name(int nIndex);
 std::string GetParentDir(const char *szPath);
@@ -95,6 +100,11 @@ int FormatIO_HTML::Load(const char *szFile, NoteDocument &doc)
 	//clear old document contents
 	doc.Clear();
 
+	if(m_bNoteCenterMode){
+		m_bAllowUnescapedInPreTag = true;
+		m_bBRTagUsed = false;
+	}
+
 	IOProcess objFile(MODE_READ);
 	
 	// attach file read layer
@@ -106,11 +116,13 @@ int FormatIO_HTML::Load(const char *szFile, NoteDocument &doc)
 
 	char szBuffer[1024];
 	int nRead;
+	INT64 nReadTotal = 0;
 
 	//TOFIX better error handling
 	bool bContinue = true;
 	while(bContinue)
 	{
+		ProgressDlg::SetCurrentSize(nReadTotal);
 		RunLoop();	//enable progress painting events
 
 		//NOTE: single layer, no need to Finalize
@@ -118,6 +130,7 @@ int FormatIO_HTML::Load(const char *szFile, NoteDocument &doc)
 		nRead = objFile.Read(szBuffer, sizeof(szBuffer), bContinue);
 		if(nRead > 0){
 			Parse(szBuffer, nRead);
+			nReadTotal += nRead;
 		}
 	}
 
@@ -228,6 +241,13 @@ void FormatIO_HTML::WriteDocContents(IOProcess &objFile)
 	g_objIni.GetValue("Export", "TOCDepth", g_nTocDepth, 1);
 	g_objIni.GetValue("Export", "ConvertImageLinks", g_bConvertImageLinks);
 
+	//calculate tab replacement string
+	int nTabSize;
+	g_objIni.GetValue("Export", "TabSize", nTabSize, 3);
+	g_strTabReplacement = "";
+	for(int i=0; i<nTabSize; i++)
+		g_strTabReplacement += "&nbsp;";
+
 	if(m_bHtmlExport)
 	{
 		if(m_pDoc->m_strCustomCSS.size() > 0)
@@ -414,10 +434,6 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 				char *szDecode = new char[nBufSize];
 				//TOFIX handle error
 
-				//decode the data
-				replaceall(strData, "\r\n", "");
-				replaceall(strData, "\n", "");
-
 				CBase64 convert;
 				int nSize = convert.Decode(strData.c_str(), szDecode);
 
@@ -435,11 +451,9 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 				gdk_pixbuf_ref (pixbuf); // make sure we keep a reference to it
 				g_object_unref (G_OBJECT (loader));
 
-				int nCnt = m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstPictures.size();
-
 				PixInfo info;
 				info.bUsePNG = bUsePNG;
-				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1) + nCnt;
+				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 				info.pixbuf  = pixbuf;
 				info.strName = strTitle;
 				m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstPictures.push_back(info);
@@ -650,10 +664,6 @@ void FormatIO_HTML::OnTagEnd(const char *szTag)
 				char *szDecode = new char[nBufSize];
 				//TOFIX handle error
 
-				//decode the data
-				replaceall(m_strLinkData, "\r\n", "");
-				replaceall(m_strLinkData, "\n", "");
-
 				CBase64 convert;
 				int nSize = convert.Decode(m_strLinkData.c_str(), szDecode);
 
@@ -1315,6 +1325,7 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 					const char *szBuf = strTmp.c_str();
 					gchar* pszPos = g_utf8_offset_to_pointer(szBuf, link.m_nStartOffset-nOffsetTot);
 					nPosBuf = pszPos - szBuf;
+					ASSERT(nPosBuf >= 0);
 
 					//erase original segment
 					bool bText = lstSegments[nSegmentIdx].bIsText;
@@ -1441,6 +1452,8 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 				// keep breaking text within the segments list
 				//
 
+				int nPicsBeforeThis = i;
+
 				//find segment for given offset (ignore link segments)
 				int nSegmentIdx = -1;
 				int nOffset = pix.nOffset;		
@@ -1452,7 +1465,7 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 
 					std::string &strTmp = lstSegments[j].strText;
 					int nSize = g_utf8_strlen(strTmp.c_str(), -1);
-					if(nOffsetTot + nSize > nOffset){
+					if(nOffsetTot + nSize + nPicsBeforeThis > nOffset){
 						nSegmentIdx = j;
 						break;
 					}
@@ -1467,8 +1480,9 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 					//convert character offset to the buffer offset
 					int nPosBuf;
 					const char *szBuf = strTmp.c_str();
-					gchar* pszPos = g_utf8_offset_to_pointer(szBuf, pix.nOffset-nOffsetTot-i); // i -> account for previous images (each taking one byte)
+					gchar* pszPos = g_utf8_offset_to_pointer(szBuf, pix.nOffset-nOffsetTot-nPicsBeforeThis);
 					nPosBuf = pszPos - szBuf;
+					ASSERT(nPosBuf >= 0);
 
 					//erase original segment
 					bool bText = lstSegments[nSegmentIdx].bIsText;
@@ -1519,9 +1533,9 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 
 						if(bWriteFile){
 							#if GTK_CHECK_VERSION(2,8,0) //new "compression" param TOFIX set proper version
-								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, NULL);	//TOFIX check error
+								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, (char *)NULL);	//TOFIX check error
 							#else
-								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, NULL);	//TOFIX check error
+								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, (char *)NULL);	//TOFIX check error
 							#endif
 						}
 
@@ -1536,9 +1550,9 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 						gchar *buffer = NULL;
 						gsize buffer_size = 0;
 					#if GTK_CHECK_VERSION(2,8,0) //new "compression" param TOFIX set proper version
-						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, NULL)){
+						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, (char *)NULL)){
 					#else
-						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, NULL)){
+						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, (char *)NULL)){
 					#endif
 							//TOFIX handle error
 							ASSERT(false);
@@ -1606,9 +1620,9 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 
 						if(bWriteFile){
 							#if GTK_CHECK_VERSION(2,8,0) //new "compression" param TOFIX set proper version
-								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, NULL);	//TOFIX check error	
+								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, (char *)NULL);	//TOFIX check error	
 							#else
-								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, NULL);	//TOFIX check error	
+								gdk_pixbuf_save(pix.pixbuf, szPath, pix.bUsePNG? "png" : "jpeg", NULL, (char *)NULL);	//TOFIX check error	
 							#endif
 						}
 
@@ -1621,9 +1635,9 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 						gchar *buffer = NULL;
 						gsize buffer_size = 0;
 					#if GTK_CHECK_VERSION(2,8,0) //new "compression" param TOFIX set proper version
-						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, NULL)){
+						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, pix.bUsePNG? "compression" : NULL,  pix.bUsePNG? "9" : NULL, (char *)NULL)){
 					#else
-						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, NULL)){
+						if(!gdk_pixbuf_save_to_buffer(pix.pixbuf, &buffer, &buffer_size, pix.bUsePNG? "png" : "jpeg", NULL, (char *)NULL)){
 					#endif
 							//TOFIX handle error
 							ASSERT(false);
@@ -1661,6 +1675,13 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 			{
 				FmtInfo &fmt = node.m_lstTxtFmt[i];
 
+				int nPicsBeforeThis = 0;
+				for(int k=0; k<nPicsCount; k++)
+					if(node.m_lstPictures[k].nOffset < fmt.nOffset)
+						nPicsBeforeThis ++;
+					else
+						break;
+
 				//
 				// keep breaking text within the segments list
 				//
@@ -1676,7 +1697,7 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 
 					std::string &strTmp = lstSegments[j].strText;
 					int nSize = g_utf8_strlen(strTmp.c_str(), -1);
-					if(nOffsetTot + nSize > nOffset){
+					if(nOffsetTot + nSize + nPicsBeforeThis > nOffset){
 						nSegmentIdx = j;
 						break;
 					}
@@ -1691,8 +1712,9 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 					//convert character offset to the buffer offset
 					int nPosBuf;
 					const char *szBuf = strTmp.c_str();
-					gchar* pszPos = g_utf8_offset_to_pointer(szBuf, fmt.nOffset-nOffsetTot);
+					gchar* pszPos = g_utf8_offset_to_pointer(szBuf, fmt.nOffset-nOffsetTot-nPicsBeforeThis);
 					nPosBuf = pszPos - szBuf;
+					ASSERT(nPosBuf >= 0);
 
 					//erase original segment
 					bool bText = lstSegments[nSegmentIdx].bIsText;
@@ -1819,6 +1841,12 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 				replaceall(strTmp, "\r\n", "\r");
 				replaceall(strTmp, "\n", "\r");
 				replaceall(strTmp, "\r", "<BR>\n");
+
+				if(bHtmlExport){
+					//convert spaces to non-breaking space
+					replaceall(strTmp, " ", "&nbsp;");
+					replaceall(strTmp, "\t", g_strTabReplacement.c_str());
+				}
 			}
 			strData += strTmp;
 		}
