@@ -20,6 +20,7 @@
 #include "FormatIOExecutable.h"
 #include "DocumentIterator.h"
 #include "FilePath.h"
+#include "File64.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -338,6 +339,7 @@ int NoteDocument::Load(const char *szPath, const char *szPassword, int nFormat)
 	//show wait dialog (only if not from cmd line)
 #ifndef _NOKIA_MAEMO	
 	ProgressDlg dlg(200, _("Loading file. Please wait..."), window1);
+	dlg.SetTotalSize(File64::FileSize(szPath));
 #endif
 
 	if(FORMAT_EXECUTABLE != nFormat)
